*---------------------------------------------------------------------*
*
* Program........: HHAGEN.PRG
* Function.......: Drives the Agents Maintenance Form.  This module 
*                  contains the logic for field level and read
*                  level WHEN, VALID, DEACTIVATE...etc. Clauses
* Tables.........: HHAGEN.DBF
* Relations......: NONE
* Author.........: Ned Epstein 
* Creation Date..: Wed  05-10-95
* Copyright......: (C) Ned Epstein 1995
* Calls..........: HHAgen.SPR
* The following check boxes are checked in the 
*                  edit screen set dialog for HHAgen.SPR.
*  
* 1) Define Windows
* 2) Border for GETS
* 3) Cycle Through Fields
*
* The following check boxes are UN-checked in the 
*                  edit screen set dialog.
*
* 1) Open Files
* 2) Close Files
* 2) Release Windows 
* 3) Modal Screens
* 4) READ NOLOCK
* 5) Execute Multiple Reads
* 6) Windows Objects Only
*
*---------------------------------------------------------------------*
*    TO DO LIST: Mon  04-30-95
*
IF .f.
  ? "(C) 1995 by Ned Epstein, All Rights Reserved."
ENDIF

SET STATUS OFF
on key label ctrl+f12 SUSP
* Misc constants
#DEFINE dcBell    7
#DEFINE dcPgUp    5
#DEFINE dcPgDn    18
* Button bar commands
#DEFINE dcTop     1
#DEFINE dcPrev    2
#DEFINE dcNext    3
#DEFINE dcBottom  4
#DEFINE dcList    5
#DEFINE dcFind    6
#DEFINE dcAdd     7
#DEFINE dcChange  8
#DEFINE dcUndo    9
#DEFINE dcExit    10 
* Add/Edit mode buttons
#DEFINE dcSave    1
#DEFINE dcCancel  2
* SysTask constants 
#DEFINE dcHHAgen  2 && Agent Task info is stored in row 2
#DEFINE dcWName   2 && Window name stored in column 2
#DEFINE dcCurRec  3 && Current record of suspended task in column 3
#DEFINE dcSuspend 4 && Window suspend switch in column 4 
#DEFINE dcSEdit   5 && Edit mode switch in column 5 
#DEFINE dcSAdd    6 && Add mode switch in column 6 
#DEFINE dcCurObj  7 && Current Object

PUBLIC lcACode, m.AgtName, m.Company, m.Phone
  
*-- Local memory variables
PRIVATE llEdit       && Edit mode boolean switch
PRIVATE llExit       && Exit boolean switch
PRIVATE llAdd        && Add mode boolean switch
PRIVATE lcMode       && Current mode of form Add,Edit or Browse

PRIVATE lcKey        && Key field for adds and searches
PRIVATE lcMessage    && Message for prompts
PRIVATE lcAnswer     && Return value for prompt

PRIVATE lcFldVar     && Name of current get object
PRIVATE lnPlaceHold  && Current record
PRIVATE lnAddEdit    && Add or Edit mode command push button variable name
PRIVATE lnChoice     && Browse mode command push botton variable name
PRIVATE llDeleted    && Value of deleted mark for current record

*-- Init vars 
glAgenOpt     = .t. && disable menu option  
lcLaunchIt    = []  && prepare for next launch  
lcKey         = [ ]
lnChoice      = 0
lnAddEdit     = 0
llEdit        = .f.
llAdd         = .f.
llExit        = .f.
lcAnswer      = [ ]
lcMessage     = [ ]
lcfldvar      = [ ]

SELECT Agen 
SET ORDER TO TAG ACode   
GO BOTTOM
lnLastRec=RECNO()
GOTO TOP
lnFirstRec=RECNO()
lnPlaceHold=lnFirstRec
SET EXACT ON
* Restore environment
IF gaSysTask(dcHHAgen,dcSuspend)  
  *-- if not zero then code was suspended
  lnPlaceHold = gaSysTask(dcHHAgen,dcCurRec)
  GOTO lnPlaceHold
  llAdd=gaSysTask(dcHHAgen,dcSAdd)
  llEdit=gaSysTask(dcHHAgen,dcSEdit)
ELSE && if not suspended then reinit vars
  DO lpMemVar WITH "FILL"
ENDIF  
DO CASE
  CASE llEdit
    lcMode = [Change mode; Press PgDn to save]
  CASE llAdd
    lcMode = [Add mode; Press PgDn to save]
  OTHERWISE
    lcMode = [Browse mode]
ENDCASE
*SET STEP ON
DO HHAgen.SPR
RETURN

*-----------------------------------------------------------------------*
*                       End Of Program
*-----------------------------------------------------------------------*
 
*-----------------------------------------------------------------------*
*                       Procedures and Functions
*-----------------------------------------------------------------------*

*-----------------------------------------------------------------------*
*
* NAME...........: lpSiteBrw 
* Function.......: List records in Agent table to a browse window to
*                  allow user selection. 
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
PROCEDURE lpAgenBrw 
  IF !WEXIST([wAgenBrw])
    DEFINE WINDOW wAgenBrw AT 1,1 SIZE 14,80 ;
       FONT [MS Sans Serif],8 STYLE [B] FLOAT CLOSE 
    MOVE WINDOW wAgenBrw CENTER
  ENDIF
  lcMessage=[Highlight to select. Enter to return]
  SET MESSAGE TO lcMessage
  ON KEY LABEL Enter KEYBOARD '{CTRL+W}' PLAIN
  BROWSE FIELDS aCode :H=[Agent Code], AgtName :H=[Name],;
         Phone :H=[Phone Number];
         REST NOMENU NOMODIFY NODELETE;
         WINDOW wAgenBrw TITLE [Agents List] ; 
         WHEN lpAgenSel()
  ON KEY LABEL Enter 
  SET MESSAGE TO  
  DEACTIVATE WINDOW wAgenBrw
  RELEASE WINDOW [Agents List]
RETURN
FUNCTION lpAgenSel
  SHOW WINDOW [Agents List] REFRESH
  DO lpMemVar WITH [FILL]
  DO lpDspChc  
  SHOW GETS
RETURN .t.

*-----------------------------------------------------------------------*
*
* NAME...........: lpFW
* Function.......: This Logic is defines the field level WHEN clause.
*                  It determines which fields are allowed to be selected.  
* Inputs.........: Global; lcFldVar, Name of current object
* Outputs........: Returns Boolean, True or False
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpFW && Field When
PRIVATE llRetVal
llRetVal=.f.
lcFldVar=VARREAD()
lcFldVar=UPPER(lcFldVar)
DO CASE
  CASE lcFldVar = [LCACODE] && Never allow edit, key field
    llRetVal=.f.
  CASE lcFldVar = [LLDELETED] && Never allow edit, deleted status
    llRetVal=.f.
  CASE lcFldVar = [LCMODE] && Never allow edit 
    llRetVal=.f.
  CASE lcFldVar = [LNCHOICE] && Must be enabled, main command buttons
    * NOTA BENE; this case is included as refence only to aid in 
    * following the logic.  If enabled this object should always be 
    * selected, and if disabled it should not be selected and this 
    * routine would not be called.  It is disabled when llAdd or llEdit
    * is .t. (True)  
    * If enabled force to true
    llRetVal=.t. 
  OTHERWISE && in all other fields allow select for adds and edits
    IF llEdit .OR. llAdd
      llRetVal = .t.
    ENDIF  
ENDCASE
RETURN llRetVal

*-----------------------------------------------------------------------*
*
* NAME...........: lpFV
* Function.......: This defines the field level VALID clause.  Its 
*                  primary function is to drive the command button
*                  objects.  It is also used to validate data fields
* Inputs.........: Global; lcFldVar, Name of current object
* Outputs........: Always returns true
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpFV && Field Valid
PARAMETERS lcFldVar

*-- this logic for screen calls 
IF ! TYPE('lcFldVar') = 'C' .OR. EMPTY(lcFldVar)
  lcFldVar = VARREAD()
ENDIF

lcFldVar = UPPER(lcFldVar)

DO CASE
  CASE lcFldVar = [LNADDEDIT]
    DO CASE
      CASE LNADDEDIT = dcSave
        IF lpValDat() && Validate fields
          =lpSavDat()
          * Recompute first and last even after failed record lock
          lnPlaceHold = RECNO()
          GOTO TOP
          lnFirstRec = RECNO()
          GOTO BOTTOM
          lnLastRec = RECNO()
          GOTO lnPlaceHold
          STORE .f. TO llAdd, llEdit 
          lcMode = [Browse mode]
          SHOW GET lnAddEdit   DISABLE
          GOTO lnPlaceHold
          DO lpDspChc  
          ON KEY LABEL PGDN  
          ON KEY LABEL ALT+F4 
        ENDIF
      CASE LNADDEDIT = dcCancel
        STORE .f. TO llAdd, llEdit 
        lcMode = [Browse mode]
        SHOW GET lnAddEdit DISABLE
        GOTO lnPlaceHold
        DO lpDspChc 
        _CUROBJ=OBJNUM(lnChoice)
        DO lpMemVar WITH [FILL]
        ON KEY LABEL PGDN  
    ENDCASE
    SHOW GETS
    * end of CASE lcFldVar = [LNADDEDIT]
  CASE lcFldVar = [LNCHOICE]
    DO CASE
      CASE lnChoice = dcTop
        GOTO TOP
        DO lpMemVar WITH 'FILL'
        DO lpDspChc
        lnPlaceHold = RECNO()
        SHOW GETS
      CASE lnChoice = dcBottom
        GOTO BOTTOM
        DO lpMemVar WITH 'FILL'
        DO lpDspChc
        lnPlaceHold = RECNO()
        SHOW GETS
      CASE lnChoice = dcNext
        *-- Next 
        IF RECNO() <> lnLastRec 
          SKIP
          DO lpMemVar WITH 'FILL'
          DO lpDspChc
          SHOW GETS
        ELSE
          ?? CHR(7) 
        ENDIF     
        lnPlaceHold = RECNO()
        SHOW GETS
      CASE lnChoice = dcPrev
        *-- Previous record
        IF RECNO() <> lnFirstRec 
          SKIP -1
          DO lpMemVar WITH 'FILL'
          DO lpDspChc
          SHOW GETS
        ELSE
          ?? CHR(7) 
        ENDIF     
        lnPlaceHold = RECNO()
        SHOW GETS
      CASE lnChoice = dcList
        DO lpAgenBrw
        lnPlaceHold = RECNO()
      CASE lnChoice = dcExit
        DO lpExit
      CASE lnChoice = dcFind
        lcKey = SPACE(6) 
        DO HHGetKey.SPR WITH lcKey,[!!!!!!],[Enter agent code to search for]
        IF LEN(ALLTRIM(lcKey)) > 0 && Check if the user entered blank
          lcKey = ALLTRIM(lcKey) 
          lnPlaceHold=RECNO()
          SEEK lcKey
          IF .NOT. FOUND()
            GOTO lnPlaceHold
            ?? CHR(7)
            WAIT WINDOW 'Agent code '+ALLTRIM(lcKey)+[ not found...]
          ELSE
            lcACode = ALLTRIM(lcKey) 
            DO lpMemVAr WITH 'FILL'
          ENDIF
        ENDIF
        SHOW GETS
      CASE lnChoice = dcAdd
        lcKey = SPACE(6) 
        DO HHGetKey.SPR WITH lcKey,[!!!!!!],[Enter new agent code]
        IF LEN(ALLTRIM(lcKey)) > 0 && Check if the user entered blank
          lcKey = ALLTRIM(lcKey) 
          llAdd = .t.
          lcMode = [Add mode; Press PgDn to save]
          ON KEY LABEL PGDN DO lpPgDn
          ON KEY LABEL ALT+F4 DO lpAgenBrw
          lnPlaceHold=RECNO()
          SEEK lcKey
          IF FOUND()
            GOTO lnPlaceHold
            ?? CHR(7)
            WAIT WINDOW [Agent code ]+ALLTRIM(lcKey)+[ already exists...]
          ELSE
            DO lpMemVar WITH [ADD]
            lcACode=lcKey
            _CUROBJ=OBJNUM(m.AgtName)
            SHOW GET lnChoice   DISABLE        
            SHOW GET lnAddEdit   ENABLE        
          ENDIF  
        ELSE
          lcMessage=[Browse Mode]
        ENDIF
        SHOW GETS
      CASE lnChoice = dcChange 
        llEdit = .t.
        lcMode = [Change mode; Press PgDn to save]
        ON KEY LABEL PGDN DO lpPgDn
        ON KEY LABEL ALT+F4 DO lpAgenBrw
        lnPlaceHold=RECNO()
        _CUROBJ=OBJNUM(m.AgtName)
        SHOW GET lnChoice   DISABLE        
        SHOW GET lnAddEdit   ENABLE        
        SHOW GETS
      CASE lnChoice = dcUndo
        * this logic is for the delete function
        IF DELETED() 
          lcMessage=[UNdelete Record for ]+ALLTRIM(lcACode)+[ ?]
        ELSE
          lcMessage=[Delete Record for ]+ALLTRIM(lcACode)+[ ?]
        ENDIF 
        lcAnswer = [Quit] 
        DO HHDelChc.SPR
*       Keep variable lcSTemp just in case client wants DELETED ON 
        lcSTemp = lcACode 
        IF lcAnswer = [Ok] 
          SELECT Agen
          llRecLck=RLOCK()
          IF llRecLck  
            IF DELETED()
              RECALL
            ELSE
              DELETE
            ENDIF
            WAIT [Record for Agent ]+ALLTRIM(lcSTemp)+;
                 [ processed] WINDOW NOWAIT
            llRetVal = .t. 
          ELSE && in use 
            WAIT [ Record lock failed, process cancelled! ] WINDOW
            llRetVal = .f.
          ENDIF  
          DO lpMemVar WITH [FILL] 
        ENDIF 
        SHOW GETS
    ENDCASE 
    * end of CASE lcFldVar = [LNCHOICE]
ENDCASE
RETURN .T.

*-----------------------------------------------------------------------*
*
* NAME...........: lpDspChc 
* Function.......: To determine the current state off the command button
*                  buttons and set them. It check current position of the
*                  record pointer and enables and disables the appropriate
*                  buttons.
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
PROCEDURE lpDspChc
SHOW GET lnChoice ENABLE
DO CASE
  CASE RECNO() = lnFirstRec
    SHOW GET lnChoice,dcTop    DISABLE
    SHOW GET lnChoice,dcPrev   DISABLE
  CASE RECNO() = lnLastRec
    SHOW GET lnChoice,dcNext   DISABLE
    SHOW GET lnChoice,dcBottom DISABLE
ENDCASE
RETURN


*-----------------------------------------------------------------------*
*
* NAME...........: lpRW 
* Function.......: This defines the READ level WHEN clause logic.
*                  If the task is being intiated after it was suspended 
*                  the record pointer and objects are set to their 
*                  last values. 
* Inputs.........: task array values
* Called By......: HHSITE.SPR; READ 
*
*-----------------------------------------------------------------------*
FUNCTION lpRW && Read When
PRIVATE llRetVal
* When the commodity screen is initially displayed first record
* of table is diplayed. Therefore, disable first (Prev) and top 
* command buttons.
IF gaSysTask(dcHHAgen,dcCurRec) <> 0 
  lnPlaceHold = gaSysTask(dcHHAgen,dcCurRec) 
  GOTO lnPlaceHold
ENDIF
IF llAdd .OR. llEdit
*-- Task was suspended restore current state
  SHOW GET lnChoice DISABLE
  SHOW GET lnAddEdit ENABLE        
  _CUROBJ = gaSysTask(dcHHAgen,dcCurObj)
  SHOW GETS
ELSE
  DO lpDspChc
ENDIF  
llRetVal = .t.
RETURN .t.

*-----------------------------------------------------------------------*
*
* NAME...........: lpRD
* Function.......: Read DEACTIVATE logic. Save the current state of 
*                  the task to the SysTask Array.
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpRD && Read deactivate
* Save Environment 
  gaSysTask(dcHHAgen,dcSuspend) = .t.
  gaSysTask(dcHHAgen,dcCurRec)  = lnPlaceHold 
  gaSysTask(dcHHAgen,dcSAdd)    = llAdd  
  gaSysTask(dcHHAgen,dcSEdit)   = llEdit
  gaSysTask(dcHHAgen,dcCurObj)  = _CUROBJ
  RELEASE WINDOW [wAgenBrw]
  llRetVal = .t.
RETURN llRetVal

*-----------------------------------------------------------------------*
*
* NAME...........: lpExit
* Function.......: Clears READ, Initializes Systask and releases 
*                  windows and memory variables.
* Inputs.........: 
* Outputs........: 
* Called By......: 
*
*-----------------------------------------------------------------------*
PROCEDURE lpExit
  llExit = .t.
  gaSysTask(dcHHAgen,dcSuspend) = .f. 
  gaSysTask(dcHHAgen,dcCurRec)  = 0
  gaSysTask(dcHHAgen,dcSAdd)    = .f.
  gaSysTask(dcHHAgen,dcSEdit)   = .f.
  gaSysTask(dcHHAgen,dcCurObj)  = 0
  CLEAR READ  
  glAgenOpt = .f. && Enable Agent maintenance menu option
  RELEASE WINDOW [wAgenBrw]
  RELEASE WINDOW [wHHAgen]
  RELEASE lcACode, m.AgtName, m.Company, m.Phone
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpPgDn
* Function.......: To move the focus to the lnAddEdit object while in
*                  add or edit mode  
* Called By......: Page down key while in add or edit mode
*
*-----------------------------------------------------------------------*
PROCEDURE lpPgDn
_CUROBJ=OBJNUM(lnAddEdit)
SHOW GETS
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpMemVar 
* Function.......: Initializes and loads memory variables from the 
*                  current record.
* Called By......: HHsite, lpFV, lpSiteBrw
*
*-----------------------------------------------------------------------*
PROCEDURE lpMemVar
PARAMETERS lcAction
DO CASE
  CASE lcAction = 'ADD'  
    SELECT Agen 
    SCATTER MEMVAR BLANK 
    STORE .f. TO llDeleted 
  CASE lcAction = 'FILL'
    SELECT Agen 
    SCATTER MEMVAR
    STORE m.ACode   TO lcACode
    STORE DELETED() TO llDeleted 
  OTHERWISE
    WAIT 'INVALID ACTION PASSED'+lcAction WINDOW    
ENDCASE
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpValDat 
* Function.......: Validates fields before record is saved.
* Outputs........: Boolean true or false
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
FUNCTION lpValDat
PRIVATE llRetVal
llRetVal = .t.
m.AgtName = ALLTRIM(m.AgtName)
IF LEN(ALLTRIM(m.AgtName))=0
  llRetVal = .f.
  _curobj = OBJNUM(m.AgtName)
  SHOW GETS
  ?? CHR(dcBell)
  WAIT WINDOW [Error, Agent name must not be blank!] NOWAIT
ENDIF
RETURN llRetval

*-----------------------------------------------------------------------*
*
* NAME...........: lpSaveDat 
* Function.......: Locks record so that changes may be saved, if add mode
*                  then new record is added and locked.
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
FUNCTION lpSavDat
PRIVATE llRecLck, llRetVal
DO CASE
  CASE llAdd
    *-- lock record add
    SELECT Agen
    APPEND BLANK
    llRecLck=RLOCK()
    IF llRecLck
      REPLACE ACode    WITH lcACode
      REPLACE AgtName    WITH m.AgtName 
      REPLACE Company  WITH m.Company 
      REPLACE Phone    WITH m.Phone
      UNLOCK
      WAIT 'Information Saved' WINDOW  NOWAIT
      llRetVal = .t. 
    ELSE
      WAIT ' Record lock failed, save cancelled! ' WINDOW
      llRetVal = .f.
    ENDIF  
  CASE llEdit
    *| NE lock record update
    SELECT Agen
    llRecLck=RLOCK()
    IF llRecLck  
      REPLACE AgtName    WITH m.AgtName 
      REPLACE Company  WITH m.Company 
      REPLACE Phone    WITH m.Phone
      UNLOCK
      WAIT 'Information Saved' WINDOW  NOWAIT
      llRetVal = .t. 
    ELSE && in use 
      WAIT ' Record lock failed, save cancelled! ' WINDOW
      llRetVal = .f.
    ENDIF  
ENDCASE
RETURN llRetVal
*-----------------------------------------------------------------------*





