*---------------------------------------------------------------------*
*
* Program........: HHSITE.PRG
* Function.......: Drives the Sites Maintenance Form.  This module 
*                  contains the logic for field level and read
*                  level WHEN, VALID, DEACTIVATE...etc. Clauses
* Tables.........: HHSITE.DBF
* Relations......: 1-to-1 relation by agent code into HHAGEN.DBF
* Author.........: Ned Epstein 
* Creation Date..: Wed  05-10-95
* Copyright......: (C) Ned Epstein 1995
* Calls..........: HHSite.SPR
* The following check boxes are checked in the 
*                  edit screen set dialog for HHSITE.SPR.
*  
* 1) Define Windows
* 2) Border for GETS
* 3) Cycle Through Fields
*
* The following check boxes are UN-checked in the 
*                  edit screen set dialog.
*
* 1) Open Files
* 2) Close Files
* 2) Release Windows 
* 3) Modal Screens
* 4) READ NOLOCK
* 5) Execute Multiple Reads
* 6) Windows Objects Only
*
*---------------------------------------------------------------------*
*    TO DO LIST: Mon  04-30-95
*
*---------------------------------------------------------------------*
IF .f.
  ? "(C) 1995 by Ned Epstein, All Rights Reserved."
ENDIF

SET STATUS OFF
on key label ctrl+f12 SUSP
* Misc constants
#DEFINE dcEsc     27
#DEFINE dcBell    7
#DEFINE dcPgUp    5
#DEFINE dcPgDn    18
* Button bar objects
#DEFINE dcTop     1
#DEFINE dcPrev    2
#DEFINE dcNext    3
#DEFINE dcBottom  4
#DEFINE dcList    5
#DEFINE dcFind    6
#DEFINE dcAdd     7
#DEFINE dcChange  8
#DEFINE dcUndo    9
#DEFINE dcExit    10 
* Add/Edit button objects
#DEFINE dcSave    1
#DEFINE dcCancel  2
* SysTask constants 
#DEFINE dcHHSite  1 && Site Task info is stored in row 1
#DEFINE dcWName   2 && Window name stored in column 2
#DEFINE dcCurRec  3 && Current record of suspended task in column 3
#DEFINE dcSuspend 4 && Window suspend switch in column 4 
#DEFINE dcSEdit   5 && Edit mode switch in column 5 
#DEFINE dcSAdd    6 && Add mode switch in column 6 
#DEFINE dcCurObj  7 && Current Object

*-- These are the form variables. they made public so that their 
*-- contents are not lost when switching windows in add or edit modes.
PUBLIC lcSCode, m.Address, m.AgentId, m.Ask, m.Bid, m.LotSize,;
       m.Bed, m.Bath, m.Rooms, m.Age, m.Taxes, lcAgtName,;
       lcAgtPhone, m.Comments, m.Location, m.Fuel  

PUBLIC ARRAY laLocation(5), laFuel(3)
STORE [Midland Park]       TO laLocation(1)
STORE [Delmar Acres]       TO laLocation(2)
STORE [Nottingham Heights] TO laLocation(3)
STORE [Belleaire Estates]  TO laLocation(4)
STORE [Parkview]           TO laLocation(5)
STORE [Oil]                TO laFuel(1)
STORE [Gas]                TO laFuel(2)
STORE [Electric]           TO laFuel(3)

*-- Local memory variables
PRIVATE llEdit       && Edit mode boolean switch
PRIVATE llExit       && Exit boolean switch
PRIVATE llAdd        && Add mode boolean switch
PRIVATE lcMode       && Current mode of form Add,Edit or Browse

PRIVATE lcKey        && Key field for adds and searches
PRIVATE lcMessage    && Message for prompts
PRIVATE lcAnswer     && Return value for prompt

PRIVATE lcFldVar     && Name of current get object
PRIVATE lnPlaceHold  && Current record
PRIVATE lnAddEdit    && Add or Edit mode command push button variable name
PRIVATE lnChoice     && Browse mode command push botton variable name
PRIVATE llDeleted    && Value of deleted mark for current record
PRIVATE lnAgtList    && Agent List button 

*-- Init vars 
glSiteOpt     = .t. && disable menu option  
lcLaunchIt    = []  && prepare for next launch  
lcKey         = [ ]
lnChoice      = 0
lnAddEdit     = 0
llEdit        = .f.
llAdd         = .f.
llExit        = .f.
lcAnswer      = [ ]
lcMessage     = [ ]
lcfldvar      = [ ]

*-- Set tags and relations 
SELECT Agen 
SET ORDER TO TAG ACode   
SELECT Site 
SET ORDER TO TAG Scode
SET RELATION TO AgentId INTO Agen 
*-- Find beginning and end of table
GO BOTTOM
lnLastRec=RECNO()
GOTO TOP
lnFirstRec=RECNO()
lnPlaceHold=lnFirstRec

SET EXACT ON
IF gaSysTask(dcHHSite,dcSuspend)  
*-- Restore environment
  lnPlaceHold = gaSysTask(dcHHSite,dcCurRec)
  GOTO lnPlaceHold
  llAdd=gaSysTask(dcHHSite,dcSAdd)
  llEdit=gaSysTask(dcHHSite,dcSEdit)
ELSE && if not suspended then reinit vars
*-- not suspended
  lcAgtName     = SPACE(LEN(Agen.AgtName)) 
  lcAgtPhone    = SPACE(LEN(Agen.Phone))
  DO lpMemVar WITH "FILL"
ENDIF  
DO CASE
  CASE llEdit
    lcMode = [Change mode; Press PgDn to save]
  CASE llAdd
    lcMode = [Add mode; Press PgDn to save]
  OTHERWISE
    lcMode = [Browse mode]
ENDCASE
*-- Call form 
DO HHSite.SPR
RETURN

*-----------------------------------------------------------------------*
*                       End Of Program
*-----------------------------------------------------------------------*
 
*-----------------------------------------------------------------------*
*                       Procedures and Functions
*-----------------------------------------------------------------------*

*-----------------------------------------------------------------------*
*
* NAME...........: lpAgenBrw 
* Function.......: Lists the agent table to a browse window to allow 
*                  selection.
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
PROCEDURE lpAgenBrw
  IF !WEXIST([wAgenBrw])
    DEFINE WINDOW wAgenBrw AT 1,1 SIZE 10,60; 
      FONT [MS Sans Serif],8 STYLE [B] FLOAT  
    MOVE WINDOW wAgenBrw CENTER
  ENDIF
  SELECT * FROM Agen INTO CURSOR a_Tcur1 ORDER BY Acode
  SELECT a_Tcur1
  lcMessage = [Highlight and press Enter. Esc to Cancel]
  SET MESSAGE TO lcMessage
  ON KEY LABEL Enter KEYBOARD '{CTRL+W}' PLAIN
  BROWSE FIELDS ACode :H=[Agent Code], AgtName :H=[Agent Name] ;
         NOMENU NORGRID NOMODIFY NODELETE ;
         WINDOW wAgenBrw TITLE [Select Agent Code]    
  ON KEY LABEL Enter
  SET MESSAGE TO
  IF LASTKEY() <> dcEsc
    m.AgentId=a_Tcur1.Acode
    lcAgtName=a_Tcur1.AgtName
    lcAgtPhone=a_Tcur1.Phone
    SHOW GET m.AgentId
    SHOW GET lcAgtCode
    SHOW GET lcAgtName
    SHOW GET lcAgtPhone
  ENDIF
  USE
  SELECT Site 
  DEACTIVATE WINDOW [Select Agent Code]
  DEACTIVATE WINDOW wAgenBrw
  RELEASE WINDOW [Select Agent Code]
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpSiteBrw 
* Function.......: List records in Site table to a browse window to
*                  allow user selection. 
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
PROCEDURE lpSiteBrw 
  IF !WEXIST([wSiteBrw])
    DEFINE WINDOW wSiteBrw AT 1,1 SIZE 14,80 ;
       FONT [MS Sans Serif],8 STYLE [B] FLOAT CLOSE 
    MOVE WINDOW wSiteBrw CENTER
  ENDIF
  lcMessage=[Highlight to select. Enter to return]
  SET MESSAGE TO lcMessage
  ON KEY LABEL Enter KEYBOARD '{CTRL+W}' PLAIN
  BROWSE FIELDS SCode :H=[Site Code], Address :H=[Address],;
         Agen.AgtName :H=[Agent];
         REST NOMENU NOMODIFY NODELETE;
         WINDOW wSiteBrw TITLE [Sites List] ; 
         WHEN lpSiteSel()
  ON KEY LABEL Enter  
  SET MESSAGE TO  
  DEACTIVATE WINDOW wSiteBrw
  RELEASE WINDOW [Sites List]
RETURN  
* Load memory variables
FUNCTION lpSiteSel
  SHOW WINDOW [Sites List] REFRESH
  DO lpMemVar WITH [FILL]
  DO lpDspChc  
  SHOW GETS
RETURN .t.

*-----------------------------------------------------------------------*
*
* NAME...........: lpFW
* Function.......: This Logic is defines the field level WHEN clause.
*                  It determines which fields are allowed to be selected.  
* Inputs.........: Global; lcFldVar, Name of current object
* Outputs........: Returns Boolean, True or False
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpFW && Field When
PRIVATE llRetVal
llRetVal=.f.
lcFldVar=VARREAD()
lcFldVar=UPPER(lcFldVar)
DO CASE
  CASE lcFldVar = [LCSCODE] && Never allow edit, key field
    llRetVal=.f.
  CASE lcFldVar = [LCAGTNAME] && Never allow edit 
    llRetVal=.f.
  CASE lcFldVar = [LCAGTPHONE] && Never allow edit 
    llRetVal=.f.
  CASE lcFldVar = [LLDELETED] && Never allow edit, deleted status
    llRetVal=.f.
  CASE lcFldVar = [LCMODE] && Never allow edit 
    llRetVal=.f.
  CASE lcFldVar = [LNCHOICE] && Must be enabled, main command buttons
    * NOTA BENE; this case is included as refence only to aid in 
    * following the logic.  If enabled this object should always be 
    * selected, and if disabled it should not be selected and this 
    * routine would not be called.  It is disabled when llAdd or llEdit
    * is .t. (True)  
    * If enabled force to true
    llRetVal=.t. 
  OTHERWISE && in all other fields allow select for adds and edits
    IF llEdit .OR. llAdd
      llRetVal = .t.
    ENDIF  
ENDCASE
RETURN llRetVal

*-----------------------------------------------------------------------*
*
* NAME...........: lpFV
* Function.......: This defines the field level VALID clause.  Its 
*                  primary function is to drive the command button
*                  objects.  It is also used to validate data fields
* Inputs.........: Global; lcFldVar, Name of current object
* Outputs........: Always returns true
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpFV && Field Valid
PARAMETERS lcFldVar

*-- this logic for screen calls 
IF ! TYPE('lcFldVar') = 'C' .OR. EMPTY(lcFldVar)
  lcFldVar = VARREAD()
ENDIF

lcFldVar = UPPER(lcFldVar)

DO CASE
  CASE lcFldVar = [LNADDEDIT]
    DO CASE
      CASE LNADDEDIT = dcSave
        IF lpValDat() && Validate fields
          =lpSavDat()
          * Recompute first and last even after failed record lock
          lnPlaceHold = RECNO()
          GOTO TOP
          lnFirstRec = RECNO()
          GOTO BOTTOM
          lnLastRec = RECNO()
          GOTO lnPlaceHold
          STORE .f. TO llAdd, llEdit 
          lcMode = [Browse mode]
          SHOW GET lnAddEdit   DISABLE
          GOTO lnPlaceHold
          DO lpDspChc  
          ON KEY LABEL PGDN  
          ON KEY LABEL ALT+F4 
        ENDIF
      CASE LNADDEDIT = dcCancel
        STORE .f. TO llAdd, llEdit 
        lcMode = [Browse mode]
        SHOW GET lnAddEdit DISABLE
        GOTO lnPlaceHold
        DO lpDspChc 
        _CUROBJ=OBJNUM(lnChoice)
        DO lpMemVar WITH [FILL]
        ON KEY LABEL PGDN  
    ENDCASE
    SHOW GETS
    * end of CASE lcFldVar = [LNADDEDIT]
  CASE lcFldVar = [LNCHOICE]
    DO CASE
      CASE lnChoice = dcTop
        GOTO TOP
        DO lpMemVar WITH 'FILL'
        DO lpDspChc
        lnPlaceHold = RECNO()
      CASE lnChoice = dcBottom
        GOTO BOTTOM
        DO lpMemVar WITH 'FILL'
        DO lpDspChc
        lnPlaceHold = RECNO()
      CASE lnChoice = dcNext
        *-- Next 
        IF RECNO() <> lnLastRec 
          SKIP
          DO lpMemVar WITH 'FILL'
          DO lpDspChc
          SHOW GETS
        ELSE
          ?? CHR(7) 
        ENDIF     
        lnPlaceHold = RECNO()
      CASE lnChoice = dcPrev
        *-- Previous record
        IF RECNO() <> lnFirstRec 
          SKIP -1
          DO lpMemVar WITH 'FILL'
          DO lpDspChc
          SHOW GETS
        ELSE
          ?? CHR(7) 
        ENDIF     
        lnPlaceHold = RECNO()
      CASE lnChoice = dcList
        DO lpSiteBrw
        lnPlaceHold = RECNO()
      CASE lnChoice = dcExit
        DO lpExit
      CASE lnChoice = dcFind
        lcKey = SPACE(6) 
        DO HHGetKey.SPR WITH lcKey,[!!!!!!],[Enter site to search for]
        IF LEN(ALLTRIM(lcKey)) > 0 && Check if the user entered blank
          lcKey = ALLTRIM(lcKey) 
          lnPlaceHold=RECNO()
          SEEK lcKey
          IF .NOT. FOUND()
            GOTO lnPlaceHold
            ?? CHR(7)
            WAIT WINDOW 'Site code '+ALLTRIM(lcKey)+[ not found...]
          ELSE
            lcSCode = ALLTRIM(lcKey) 
            DO lpMemVAr WITH 'FILL'
          ENDIF
        ENDIF
      CASE lnChoice = dcAdd
        lcKey = SPACE(6) 
        DO HHGetKey.SPR WITH lcKey, [!!!!!!], [Enter new site code]
        IF LEN(ALLTRIM(lcKey)) > 0 && Check if the user entered blank
          lcKey = ALLTRIM(lcKey) 
          llAdd = .t.
          lcMode = [Add mode; Press PgDn to save]
          ON KEY LABEL PGDN DO lpPgDn
          ON KEY LABEL ALT+F4 DO lpAgenBrw
          lnPlaceHold=RECNO()
          SEEK lcKey
          IF FOUND()
            GOTO lnPlaceHold
            ?? CHR(7)
            WAIT WINDOW [Site code ]+ALLTRIM(lcKey)+[ already exists...]
          ELSE
            DO lpMemVar WITH [ADD]
            lcSCode=lcKey
            _CUROBJ=OBJNUM(m.Address)
            SHOW GET lnChoice   DISABLE        
            SHOW GET lnAddEdit   ENABLE        
          ENDIF  
        ELSE
          lcMessage=[Browse Mode]
        ENDIF
      CASE lnChoice = dcChange 
        llEdit = .t.
        lcMode = [Change mode; Press PgDn to save]
        ON KEY LABEL PGDN DO lpPgDn
        ON KEY LABEL ALT+F4 DO lpAgenBrw
        lnPlaceHold=RECNO()
        _CUROBJ=OBJNUM(m.Address)
        SHOW GET lnChoice   DISABLE        
        SHOW GET lnAddEdit   ENABLE        
      CASE lnChoice = dcUndo
        * this logic is for the delete function
        IF DELETED() 
          lcMessage=[UNdelete Record for ]+ALLTRIM(lcSCode)+[ ?]
        ELSE
          lcMessage=[Delete Record for ]+ALLTRIM(lcSCode)+[ ?]
        ENDIF 
        lcAnswer = [Quit] 
        DO HHDelChc.SPR
*       Keep variable lcSTemp just in case client wants DELETED ON 
        lcSTemp = lcSCode 
        IF lcAnswer = [Ok] 
          SELECT Site
          llRecLck=RLOCK()
          IF llRecLck  
            IF DELETED()
              RECALL
            ELSE
              DELETE
            ENDIF
            WAIT [Record for Site ]+ALLTRIM(lcSTemp)+;
                 [ processed] WINDOW NOWAIT
            llRetVal = .t. 
          ELSE && in use 
            WAIT [ Record lock failed, process cancelled! ] WINDOW
            llRetVal = .f.
          ENDIF  
          DO lpMemVar WITH [FILL] 
        ENDIF 
    ENDCASE 
    * end of CASE lcFldVar = [LNCHOICE]
    IF  !llExit && if llExit is true then gets have been cleared 
      SHOW GETS
    ENDIF
  CASE lcFldVar = [LNAGTLIST]
    DO lpAgenBrw
  CASE lcFldVar = [AGENTID]
    * This logic validates the agentId field in the HHSite.SPR
    IF LEN(ALLTRIM(m.AgentId)) = 0
      DO lpAgenBrw   
    ELSE 
      SELECT * FROM Agen INTO CURSOR a_Tcur1 WHERE Agen.aCode=m.AgentId
      SELECT a_Tcur1
      USE 
      SELECT SITE
      IF _TALLY = 0
        ?? CHR(7)
        WAIT WINDOW [Invalid Agent Id.  Please select from list] NOWAIT
        DO lpAgenBrw   
      ENDIF
    ENDIF
ENDCASE
RETURN .T.

* Control display of buttons
*-----------------------------------------------------------------------*
*
* NAME...........: lpDspChc 
* Function.......: To determine the current state off the command button
*                  buttons and set them. It check current position of the
*                  record pointer and enables and disables the appropriate
*                  buttons.
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
PROCEDURE lpDspChc
SHOW GET lnChoice ENABLE
DO CASE
  CASE RECNO() = lnFirstRec
    SHOW GET lnChoice,dcTop    DISABLE
    SHOW GET lnChoice,dcPrev   DISABLE
  CASE RECNO() = lnLastRec
    SHOW GET lnChoice,dcNext   DISABLE
    SHOW GET lnChoice,dcBottom DISABLE
ENDCASE
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpRW 
* Function.......: This defines the READ level WHEN clause logic.
*                  If the task is being intiated after it was suspended 
*                  the record pointer and objects are set to their 
*                  last values. 
* Inputs.........: task array values
* Called By......: HHSITE.SPR; READ 
*
*-----------------------------------------------------------------------*
FUNCTION lpRW && Read When
PRIVATE llRetVal
* When the commodity screen is initially displayed first record
* of table is diplayed. Therefore, disable first (Prev) and top 
* command buttons.
IF gaSysTask(dcHHSite,dcCurRec) <> 0 
  lnPlaceHold = gaSysTask(dcHHSite,dcCurRec) 
  GOTO lnPlaceHold
ENDIF
IF llAdd .OR. llEdit
*-- Task was suspended restore current state
  SHOW GET lnChoice DISABLE
  SHOW GET lnAddEdit ENABLE        
  _CUROBJ = gaSysTask(dcHHSite,dcCurObj)
  SHOW GETS
ELSE
  DO lpDspChc
ENDIF  
llRetVal = .t.
RETURN .t.

*-----------------------------------------------------------------------*
*
* NAME...........: lpRD
* Function.......: Read DEACTIVATE logic. Save the current state of 
*                  the task to the SysTask Array.
* Called By......: HHSITE.SPR
*
*-----------------------------------------------------------------------*
FUNCTION lpRD && Read deactivate
* Save Environment 
  gaSysTask(dcHHSite,dcSuspend) = .t.
  gaSysTask(dcHHSite,dcCurRec)  = lnPlaceHold 
  gaSysTask(dcHHSite,dcSAdd)    = llAdd  
  gaSysTask(dcHHSite,dcSEdit)   = llEdit
  gaSysTask(dcHHSite,dcCurObj)  = _CUROBJ
  RELEASE WINDOW [wAgenBrw]
  RELEASE WINDOW [wSiteBrw]
  llRetVal = .t.
RETURN llRetVal

*-----------------------------------------------------------------------*
*
* NAME...........: lpExit
* Function.......: Clears READ, Initializes Systask and releases 
*                  windows and memory variables.
* Inputs.........: 
* Outputs........: 
* Called By......: 
*
*-----------------------------------------------------------------------*
PROCEDURE lpExit
  llExit = .t.
  gaSysTask(dcHHSite,dcSuspend) = .f. 
  gaSysTask(dcHHSite,dcCurRec)  = 0
  gaSysTask(dcHHSite,dcSAdd)    = .f.
  gaSysTask(dcHHSite,dcSEdit)   = .f.
  gaSysTask(dcHHSite,dcCurObj)  = 0
  CLEAR READ  
  glSiteOpt = .f. && Enable site maintenance menu option
  RELEASE WINDOW [wAgenBrw]
  RELEASE WINDOW [wSiteBrw]
  RELEASE WINDOW [wHHSite]
  RELEASE lcSCode, m.Address, m.AgentId, m.Ask, m.Bid, m.LotSize,;
          m.Bed, m.Bath, m.Rooms, m.Age, m.Taxes, lcAgtName,;
          lcAgtPhone, m.Comments, m.Location, m.Fuel 
  RELEASE laLocation, laFuel
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpPgDn
* Function.......: To move the focus to the lnAddEdit object while in
*                  add or edit mode  
* Called By......: Page down key while in add or edit mode
*
*-----------------------------------------------------------------------*
PROCEDURE lpPgDn
*-- Last field in screen
* set llSave to true to enable lnAddEdit in When clause
_CUROBJ=OBJNUM(lnAddEdit)
SHOW GETS
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpMemVar 
* Function.......: Initializes and loads memory variables from the 
*                  current record.
* Called By......: HHsite, lpFV, lpSiteBrw
*
*-----------------------------------------------------------------------*
PROCEDURE lpMemVar
PARAMETERS lcAction
DO CASE
  CASE lcAction = 'ADD'  
    SELECT Site 
    SCATTER MEMO MEMVAR BLANK 
    STORE SPACE(LEN(Agen.AgtName)) TO lcAgtName
    STORE SPACE(LEN(Agen.Phone)) TO lcAgtPhone
    STORE .f. TO llDeleted 
  CASE lcAction = 'FILL'
    SELECT Site 
    SCATTER MEMO MEMVAR
    STORE m.Scode           TO lcSCode
    STORE DELETED()         TO llDeleted 
    STORE LTRIM(Agen.AgtName) TO lcAgtName
    STORE LTRIM(Agen.Phone) TO lcAgtPhone
  OTHERWISE
    WAIT 'INVALID ACTION PASSED'+lcAction WINDOW    
ENDCASE
RETURN

*-----------------------------------------------------------------------*
*
* NAME...........: lpValDat 
* Function.......: Validates fields before record is saved.
* Outputs........: Boolean true or false
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
FUNCTION lpValDat
*-- need definition
PRIVATE llRetVal
llRetVal = .t.
m.Address = ALLTRIM(m.Address)
IF LEN(ALLTRIM(m.Address))=0
  llRetVal = .f.
  _curobj = OBJNUM(m.Address)
  SHOW GETS
  ?? CHR(dcBell)
  WAIT WINDOW [Error, Address must not be blank!] NOWAIT
ENDIF
RETURN llRetval

*-----------------------------------------------------------------------*
*
* NAME...........: lpSaveDat 
* Function.......: Locks record so that changes may be saved, if add mode
*                  then new record is added and locked.
* Called By......: lpFV
*
*-----------------------------------------------------------------------*
FUNCTION lpSavDat
PRIVATE llRecLck, llRetVal
DO CASE
  CASE llAdd
    *-- lock record add
    SELECT Site
    APPEND BLANK
    llRecLck=RLOCK()
    IF llRecLck
      REPLACE SCode    WITH lcSCode
      REPLACE address  WITH m.Address 
      REPLACE AgentId  WITH m.AgentId 
      REPLACE Ask      WITH m.Ask 
      REPLACE Bid      WITH m.Bid 
      REPLACE LotSize  WITH m.LotSize 
      REPLACE Bed      WITH m.Bed 
      REPLACE Bath     WITH m.Bath 
      REPLACE Rooms    WITH m.Rooms 
      REPLACE Age      WITH m.Age 
      REPLACE Taxes    WITH m.Taxes 
      REPLACE Comments WITH m.Comments
      REPLACE Location WITH m.Location
      REPLACE Fuel     WITH m.Fuel
      UNLOCK
      WAIT 'Information Saved' WINDOW  NOWAIT
      llRetVal = .t. 
    ELSE
      WAIT ' Record lock failed, save cancelled! ' WINDOW
      llRetVal = .f.
    ENDIF  
  CASE llEdit
    *| NE lock record update
    SELECT Site
    llRecLck=RLOCK()
    IF llRecLck  
      REPLACE address  WITH m.Address 
      REPLACE AgentId  WITH m.AgentId 
      REPLACE Ask      WITH m.Ask 
      REPLACE Bid      WITH m.Bid 
      REPLACE LotSize  WITH m.LotSize 
      REPLACE Bed      WITH m.Bed 
      REPLACE Bath     WITH m.Bath 
      REPLACE Rooms    WITH m.Rooms 
      REPLACE Age      WITH m.Age 
      REPLACE Taxes    WITH m.Taxes 
      REPLACE Comments WITH m.Comments
      REPLACE Location WITH m.Location
      REPLACE Fuel     WITH m.Fuel
      UNLOCK
      WAIT 'Information Saved' WINDOW  NOWAIT
      llRetVal = .t. 
    ELSE && in use 
      WAIT ' Record lock failed, save cancelled! ' WINDOW
      llRetVal = .f.
    ENDIF  
ENDCASE
RETURN llRetVal
*-----------------------------------------------------------------------*


